from __future__ import annotations

from unittest import TestCase
from unittest.mock import Mock, patch

from box import Box

from pycarlo.features.monitor import (
    AgentMonitorConfig,
    MonitorExecutionStatus,
    MonitorService,
)
from pycarlo.features.monitor.queries import (
    GET_AGENT_MONITOR_MINIMAL,
    GET_MANUAL_MONITOR_EXECUTION_STATUS,
    RUN_MONITOR,
)


class MonitorServiceTests(TestCase):
    def setUp(self) -> None:
        self._mock_client = Mock()
        self._service = MonitorService(mc_client=self._mock_client, print_func=Mock())

    def test_run_monitor_basic(self) -> None:
        """Test basic run_monitor call."""
        self._mock_client.return_value = Box(
            {
                "run_monitor": {
                    "success": True,
                    "manual_monitor_execution_uuid": "exec-uuid-123",
                }
            }
        )

        result = self._service.run_monitor(monitor_uuid="monitor-uuid-456")

        self._mock_client.assert_called_once()
        call_kwargs = self._mock_client.call_args
        self.assertEqual(call_kwargs.kwargs["query"], RUN_MONITOR)
        self.assertEqual(call_kwargs.kwargs["variables"], {"monitorUuid": "monitor-uuid-456"})
        self._assert_telemetry_headers(call_kwargs.kwargs["additional_headers"])
        self.assertEqual(result, "exec-uuid-123")

    def test_run_monitor_with_runtime_variables(self) -> None:
        """Test run_monitor with runtime variables."""
        self._mock_client.return_value = Box(
            {
                "run_monitor": {
                    "success": True,
                    "manual_monitor_execution_uuid": "exec-uuid-123",
                }
            }
        )

        result = self._service.run_monitor(
            monitor_uuid="monitor-uuid-456",
            runtime_variables={"ci_build_id": "build-789", "env": "staging"},
        )

        call_kwargs = self._mock_client.call_args
        self.assertEqual(
            call_kwargs.kwargs["variables"],
            {
                "monitorUuid": "monitor-uuid-456",
                "runtimeVariables": [
                    {"name": "ci_build_id", "value": "build-789"},
                    {"name": "env", "value": "staging"},
                ],
            },
        )
        self.assertEqual(result, "exec-uuid-123")

    def test_run_monitor_failure(self) -> None:
        """Test run_monitor raises ValueError on failure."""
        self._mock_client.return_value = Box(
            {
                "run_monitor": {
                    "success": False,
                    "manual_monitor_execution_uuid": None,
                }
            }
        )

        with self.assertRaises(ValueError) as context:
            self._service.run_monitor(monitor_uuid="monitor-uuid-456")

        self.assertIn("Failed to run monitor", str(context.exception))

    def test_get_execution_status_in_progress(self) -> None:
        """Test get_execution_status for in-progress execution."""
        self._mock_client.return_value = Box(
            {
                "get_manual_monitor_execution_status": {
                    "status": "IN_PROGRESS",
                    "breached": None,
                    "completed": False,
                }
            }
        )

        result = self._service.get_execution_status("exec-uuid-123")

        call_kwargs = self._mock_client.call_args
        self.assertEqual(call_kwargs.kwargs["query"], GET_MANUAL_MONITOR_EXECUTION_STATUS)
        self.assertEqual(
            call_kwargs.kwargs["variables"],
            {"manualMonitorExecutionUuid": "exec-uuid-123"},
        )
        self.assertIsInstance(result, MonitorExecutionStatus)
        self.assertEqual(result.status, "IN_PROGRESS")
        self.assertIsNone(result.breached)
        self.assertFalse(result.completed)

    def test_get_execution_status_success_breached(self) -> None:
        """Test get_execution_status for completed breached execution."""
        self._mock_client.return_value = Box(
            {
                "get_manual_monitor_execution_status": {
                    "status": "SUCCESS",
                    "breached": True,
                    "completed": True,
                }
            }
        )

        result = self._service.get_execution_status("exec-uuid-123")

        self.assertEqual(result.status, "SUCCESS")
        self.assertTrue(result.breached)
        self.assertTrue(result.completed)

    def test_get_execution_status_success_not_breached(self) -> None:
        """Test get_execution_status for completed non-breached execution."""
        self._mock_client.return_value = Box(
            {
                "get_manual_monitor_execution_status": {
                    "status": "SUCCESS",
                    "breached": False,
                    "completed": True,
                }
            }
        )

        result = self._service.get_execution_status("exec-uuid-123")

        self.assertEqual(result.status, "SUCCESS")
        self.assertFalse(result.breached)
        self.assertTrue(result.completed)

    @patch("pycarlo.features.monitor.service.time.sleep")
    def test_run_and_poll_success_breached(self, mock_sleep: Mock) -> None:
        """Test run_and_poll completes and returns breached status."""
        self._mock_client.side_effect = [
            Box(
                {
                    "run_monitor": {
                        "success": True,
                        "manual_monitor_execution_uuid": "exec-uuid-123",
                    }
                }
            ),
            Box(
                {
                    "get_manual_monitor_execution_status": {
                        "status": "IN_PROGRESS",
                        "breached": None,
                        "completed": False,
                    }
                }
            ),
            Box(
                {
                    "get_manual_monitor_execution_status": {
                        "status": "SUCCESS",
                        "breached": True,
                        "completed": True,
                    }
                }
            ),
        ]

        result = self._service.run_and_poll(
            monitor_uuid="monitor-uuid-456",
            runtime_variables={"ci_build_id": "build-789"},
            poll_interval_seconds=1,
        )

        self.assertTrue(result)
        mock_sleep.assert_called_once_with(1)

    @patch("pycarlo.features.monitor.service.time.sleep")
    def test_run_and_poll_success_not_breached(self, mock_sleep: Mock) -> None:
        """Test run_and_poll completes and returns not breached status."""
        self._mock_client.side_effect = [
            Box(
                {
                    "run_monitor": {
                        "success": True,
                        "manual_monitor_execution_uuid": "exec-uuid-123",
                    }
                }
            ),
            Box(
                {
                    "get_manual_monitor_execution_status": {
                        "status": "SUCCESS",
                        "breached": False,
                        "completed": True,
                    }
                }
            ),
        ]

        result = self._service.run_and_poll(
            monitor_uuid="monitor-uuid-456",
            poll_interval_seconds=1,
        )

        self.assertFalse(result)
        mock_sleep.assert_not_called()

    @patch("pycarlo.features.monitor.service.time.time")
    def test_run_and_poll_timeout(self, mock_time: Mock) -> None:
        """Test run_and_poll raises TimeoutError on timeout."""
        mock_time.side_effect = [0, 0, 700]  # Start, check, timeout exceeded

        self._mock_client.side_effect = [
            Box(
                {
                    "run_monitor": {
                        "success": True,
                        "manual_monitor_execution_uuid": "exec-uuid-123",
                    }
                }
            ),
            Box(
                {
                    "get_manual_monitor_execution_status": {
                        "status": "IN_PROGRESS",
                        "breached": None,
                        "completed": False,
                    }
                }
            ),
        ]

        with self.assertRaises(TimeoutError) as context:
            self._service.run_and_poll(
                monitor_uuid="monitor-uuid-456",
                timeout_in_minutes=10,
            )

        self.assertIn("timed out", str(context.exception))

    def test_get_agent_monitor_config_success(self) -> None:
        """Test get_agent_monitor_config returns AgentMonitorConfig with agent span filters."""
        self._mock_client.return_value = Box(
            {
                "get_monitors": [
                    {
                        "uuid": "monitor-uuid-123",
                        "name": "Test Monitor",
                        "description": "Test Description",
                        "monitor_type": "AGENT",
                        "entity_mcons": ["MCON++account++warehouse++table++schema.table"],
                        "agent_span_filters": [
                            {
                                "agent": {"value": "my-agent"},
                                "workflow": {"value": "my-workflow"},
                                "task": {"value": "my-task"},
                                "span_name": {"value": "my-span"},
                            }
                        ],
                        "filters": {
                            "type": "GROUP",
                            "operator": "AND",
                            "conditions": [
                                {
                                    "type": "BINARY",
                                    "predicate": {"name": "equal", "negated": False},
                                    "left": [{"type": "MAP_KEY", "key": "ci_build_id"}],
                                    "right": [{"type": "LITERAL", "literal": "{{ci_build_id}}"}],
                                }
                            ],
                        },
                    }
                ]
            }
        )

        result = self._service.get_agent_monitor_config("monitor-uuid-123")

        self._mock_client.assert_called_once()
        call_kwargs = self._mock_client.call_args
        self.assertEqual(call_kwargs.kwargs["query"], GET_AGENT_MONITOR_MINIMAL)
        self.assertEqual(call_kwargs.kwargs["variables"], {"uuids": ["monitor-uuid-123"]})

        self.assertIsInstance(result, AgentMonitorConfig)
        self.assertEqual(result.uuid, "monitor-uuid-123")
        self.assertEqual(result.name, "Test Monitor")
        self.assertEqual(result.description, "Test Description")
        self.assertEqual(result.monitor_type, "AGENT")
        self.assertEqual(result.mcon, "MCON++account++warehouse++table++schema.table")
        self.assertEqual(len(result.agent_span_filters), 1)
        self.assertEqual(result.agent_span_filters[0].agent, "my-agent")
        self.assertEqual(result.agent_span_filters[0].workflow, "my-workflow")
        self.assertEqual(result.agent_span_filters[0].task, "my-task")
        self.assertEqual(result.agent_span_filters[0].span_name, "my-span")
        self.assertEqual(result.runtime_variable_names, ["ci_build_id"])
        self.assertEqual(result.attribute_key, "ci_build_id")

    def test_get_agent_monitor_config_not_found(self) -> None:
        """Test get_agent_monitor_config raises ValueError when monitor not found."""
        self._mock_client.return_value = Box({"get_monitors": []})

        with self.assertRaises(ValueError) as context:
            self._service.get_agent_monitor_config("nonexistent-uuid")

        self.assertIn("Monitor not found", str(context.exception))

    def _assert_telemetry_headers(self, headers: dict[str, str]) -> None:
        self.assertEqual(headers["x-mcd-telemetry-reason"], "service")
        self.assertEqual(headers["x-mcd-telemetry-service"], "monitor_service")



from datetime import datetime
from typing import Any
from unittest import TestCase

from pycarlo.features.ingestion.models import (
    AssetField,
    AssetFreshness,
    AssetMetadata,
    AssetVolume,
    ColumnLineageField,
    ColumnLineageSourceField,
    LineageAssetRef,
    LineageEvent,
    LineageEventType,
    QueryLogEntry,
    RelationalAsset,
    Tag,
    build_lineage_payload,
    build_metadata_payload,
    build_query_log_payload,
)


class TestTag(TestCase):
    def test_to_dict_key_value(self):
        tag = Tag(key="env", value="prod")
        assert tag.to_dict() == {"key": "env", "value": "prod"}

    def test_to_dict_key_only(self):
        tag = Tag(key="pii")
        assert tag.to_dict() == {"key": "pii"}

    def test_to_dict_key_with_none_value(self):
        tag = Tag(key="sensitive", value=None)
        assert tag.to_dict() == {"key": "sensitive"}


class TestAssetField(TestCase):
    def test_to_dict_required_only(self):
        f = AssetField(name="id", type="INTEGER")
        assert f.to_dict() == {"name": "id", "type": "INTEGER"}

    def test_to_dict_with_description(self):
        f = AssetField(name="id", type="INTEGER", description="Primary key")
        assert f.to_dict() == {
            "name": "id",
            "type": "INTEGER",
            "description": "Primary key",
        }

    def test_to_dict_omits_none_description(self):
        f = AssetField(name="col", type="TEXT")
        assert "description" not in f.to_dict()


class TestAssetMetadata(TestCase):
    def test_to_dict_required_only(self):
        m = AssetMetadata(name="orders", database="analytics", schema="public")
        assert m.to_dict() == {
            "name": "orders",
            "database": "analytics",
            "schema": "public",
        }

    def test_to_dict_all_fields(self):
        m = AssetMetadata(
            name="orders",
            database="analytics",
            schema="public",
            description="Order records",
            view_query="SELECT * FROM raw.orders",
            created_on="2026-01-01T00:00:00Z",
        )
        result = m.to_dict()
        assert result["description"] == "Order records"
        assert result["view_query"] == "SELECT * FROM raw.orders"
        assert result["created_on"] == "2026-01-01T00:00:00Z"

    def test_to_dict_omits_none_optional_fields(self):
        m = AssetMetadata(name="t", database="db", schema="s")
        result = m.to_dict()
        assert "description" not in result
        assert "view_query" not in result
        assert "created_on" not in result


class TestAssetVolume(TestCase):
    def test_to_dict_with_values(self):
        v = AssetVolume(row_count=100, byte_count=2048)
        assert v.to_dict() == {"row_count": 100, "byte_count": 2048}

    def test_to_dict_partial(self):
        v = AssetVolume(row_count=50)
        assert v.to_dict() == {"row_count": 50}

    def test_to_dict_empty(self):
        v = AssetVolume()
        assert v.to_dict() == {}


class TestAssetFreshness(TestCase):
    def test_to_dict_with_value(self):
        f = AssetFreshness(last_update_time="2026-03-02T10:00:00Z")
        assert f.to_dict() == {"last_update_time": "2026-03-02T10:00:00Z"}

    def test_to_dict_empty(self):
        f = AssetFreshness()
        assert f.to_dict() == {}


class TestRelationalAsset(TestCase):
    def test_to_dict_minimal(self):
        asset = RelationalAsset(
            type="TABLE",
            metadata=AssetMetadata(name="t", database="db", schema="s"),
        )
        result = asset.to_dict()
        assert result == {
            "type": "TABLE",
            "metadata": {"name": "t", "database": "db", "schema": "s"},
        }

    def test_to_dict_omits_empty_tags_and_fields(self):
        asset = RelationalAsset(
            type="TABLE",
            metadata=AssetMetadata(name="t", database="db", schema="s"),
        )
        result = asset.to_dict()
        assert "tags" not in result
        assert "fields" not in result
        assert "volume" not in result
        assert "freshness" not in result

    def test_to_dict_full(self):
        asset = RelationalAsset(
            type="VIEW",
            metadata=AssetMetadata(
                name="v_orders",
                database="analytics",
                schema="public",
                description="Aggregated orders",
                view_query="SELECT * FROM orders",
            ),
            tags=[Tag(key="team", value="data")],
            fields=[
                AssetField(name="id", type="INTEGER"),
                AssetField(name="name", type="VARCHAR", description="Customer name"),
            ],
            volume=AssetVolume(row_count=500, byte_count=4096),
            freshness=AssetFreshness(last_update_time="2026-03-01T08:30:00Z"),
        )
        result: dict[str, Any] = asset.to_dict()

        assert result["type"] == "VIEW"
        assert result["metadata"]["name"] == "v_orders"
        assert result["metadata"]["view_query"] == "SELECT * FROM orders"
        assert result["tags"] == [{"key": "team", "value": "data"}]
        assert len(result["fields"]) == 2
        assert result["fields"][1]["description"] == "Customer name"
        assert result["volume"] == {"row_count": 500, "byte_count": 4096}
        assert result["freshness"] == {"last_update_time": "2026-03-01T08:30:00Z"}


def _dt(iso: str) -> datetime:
    """Parse ISO8601 string to datetime for tests."""
    return datetime.fromisoformat(iso.replace("Z", "+00:00"))


class TestQueryLogEntry(TestCase):
    def test_to_dict_minimal(self):
        entry = QueryLogEntry(
            start_time=_dt("2026-03-02T10:00:00Z"),
            end_time=_dt("2026-03-02T10:00:05Z"),
            query_text="SELECT * FROM orders",
        )
        result = entry.to_dict()
        assert result == {
            "start_time": "2026-03-02T10:00:00Z",
            "end_time": "2026-03-02T10:00:05Z",
            "query_text": "SELECT * FROM orders",
        }

    def test_to_dict_full(self):
        entry = QueryLogEntry(
            start_time=_dt("2026-03-02T10:00:00Z"),
            end_time=_dt("2026-03-02T10:00:05Z"),
            query_text="SELECT * FROM orders",
            query_id="q-123",
            user="analyst@example.com",
            error_code=None,
            error_text=None,
            returned_rows=1000,
            extra={"warehouse": "COMPUTE_WH"},
        )
        result = entry.to_dict()
        assert result["query_id"] == "q-123"
        assert result["user"] == "analyst@example.com"
        assert result["returned_rows"] == 1000
        assert result["extra"] == {"warehouse": "COMPUTE_WH"}
        assert "error_code" not in result
        assert "error_text" not in result

    def test_to_dict_with_error(self):
        entry = QueryLogEntry(
            start_time=_dt("2026-03-02T10:00:00Z"),
            end_time=_dt("2026-03-02T10:00:01Z"),
            query_text="SELECT * FROM invalid",
            error_code=100132,
            error_text="Object does not exist",
        )
        result = entry.to_dict()
        assert result["error_code"] == 100132
        assert result["error_text"] == "Object does not exist"

    def test_from_dict_round_trip(self):
        """Datetime fields deserialize from ISO8601 strings."""
        d = {
            "start_time": "2026-03-02T10:00:00Z",
            "end_time": "2026-03-02T10:00:05Z",
            "query_text": "SELECT 1",
        }
        entry = QueryLogEntry.from_dict(d)
        assert entry.start_time == _dt("2026-03-02T10:00:00Z")
        assert entry.end_time == _dt("2026-03-02T10:00:05Z")
        assert entry.query_text == "SELECT 1"


class TestBuildQueryLogPayload(TestCase):
    def test_payload_structure(self):
        entry = QueryLogEntry(
            start_time=_dt("2026-03-02T10:00:00Z"),
            end_time=_dt("2026-03-02T10:00:05Z"),
            query_text="SELECT 1",
        )
        payload = build_query_log_payload(
            resource_uuid="res-123",
            log_type="snowflake",
            events=[entry],
        )
        assert payload["event_type"] == "QUERY_LOG"
        assert payload["resource"] == {
            "uuid": "res-123",
            "log_type": "snowflake",
        }
        assert len(payload["events"]) == 1
        assert payload["events"][0]["start_time"] == "2026-03-02T10:00:00Z"
        assert payload["events"][0]["query_text"] == "SELECT 1"

    def test_multiple_events(self):
        events = [
            QueryLogEntry(
                start_time=_dt("2026-03-02T10:00:00Z"),
                end_time=_dt("2026-03-02T10:00:01Z"),
                query_text=f"SELECT {i}",
            )
            for i in range(3)
        ]
        payload = build_query_log_payload(
            resource_uuid="res-456",
            log_type="bigquery",
            events=events,
        )
        assert len(payload["events"]) == 3
        for i, event in enumerate(payload["events"]):
            assert event["query_text"] == f"SELECT {i}"


class TestBuildMetadataPayload(TestCase):
    def test_payload_structure(self):
        asset = RelationalAsset(
            type="TABLE",
            metadata=AssetMetadata(name="orders", database="analytics", schema="public"),
            volume=AssetVolume(row_count=1000),
        )
        payload = build_metadata_payload(
            resource_uuid="res-123",
            resource_type="snowflake",
            events=[asset],
        )

        assert payload["event_type"] == "METADATA"
        assert payload["resource"] == {
            "uuid": "res-123",
            "resource_type": "snowflake",
        }
        assert len(payload["events"]) == 1
        event = payload["events"][0]
        assert "relational_asset" in event
        assert event["relational_asset"]["type"] == "TABLE"
        assert event["relational_asset"]["volume"] == {"row_count": 1000}

    def test_multiple_events(self):
        events = [
            RelationalAsset(
                type="TABLE",
                metadata=AssetMetadata(name=f"table_{i}", database="db", schema="s"),
            )
            for i in range(3)
        ]
        payload = build_metadata_payload(
            resource_uuid="res-456",
            resource_type="bigquery",
            events=events,
        )
        assert len(payload["events"]) == 3
        for i, event in enumerate(payload["events"]):
            assert event["relational_asset"]["metadata"]["name"] == f"table_{i}"


# ---------------------------------------------------------------
# Lineage models
# ---------------------------------------------------------------


class TestLineageAssetRef(TestCase):
    def test_to_dict_required_only(self):
        ref = LineageAssetRef(type="TABLE", name="t", database="db", schema="s")
        result = ref.to_dict()
        assert result["type"] == "TABLE"
        assert result["name"] == "t"
        assert result["database"] == "db"
        assert result["schema"] == "s"

    def test_to_dict_omits_none_asset_id(self):
        ref = LineageAssetRef(type="TABLE", name="t", database="db", schema="s")
        assert "asset_id" not in ref.to_dict()

    def test_to_dict_includes_asset_id_when_set(self):
        ref = LineageAssetRef(type="TABLE", name="t", database="db", schema="s", asset_id="src1")
        assert ref.to_dict()["asset_id"] == "src1"


class TestColumnLineageSourceField(TestCase):
    def test_to_dict(self):
        f = ColumnLineageSourceField(asset_id="src1", field_name="amount")
        assert f.to_dict() == {"asset_id": "src1", "field_name": "amount"}


class TestColumnLineageField(TestCase):
    def test_to_dict(self):
        f = ColumnLineageField(
            name="total",
            source_fields=[
                ColumnLineageSourceField(asset_id="src1", field_name="amount"),
                ColumnLineageSourceField(asset_id="src2", field_name="price"),
            ],
        )
        result: dict[str, Any] = f.to_dict()
        assert result["name"] == "total"
        assert len(result["source_fields"]) == 2
        assert result["source_fields"][0] == {"asset_id": "src1", "field_name": "amount"}


class TestLineageEvent(TestCase):
    def test_to_dict_table_level(self):
        event = LineageEvent(
            destination=LineageAssetRef(
                type="TABLE", database="analytics", schema="pub", name="out"
            ),
            sources=[
                LineageAssetRef(type="TABLE", database="raw", schema="pub", name="in1"),
            ],
        )
        result: dict[str, Any] = event.to_dict()
        assert result["destination"]["name"] == "out"
        assert len(result["sources"]) == 1
        assert result["sources"][0]["name"] == "in1"

    def test_to_dict_omits_empty_fields(self):
        event = LineageEvent(
            destination=LineageAssetRef(type="TABLE", database="db", schema="s", name="t"),
            sources=[LineageAssetRef(type="TABLE", database="db", schema="s", name="s")],
        )
        assert "fields" not in event.to_dict()

    def test_to_dict_includes_fields_when_set(self):
        event = LineageEvent(
            destination=LineageAssetRef(type="TABLE", database="db", schema="s", name="t"),
            sources=[
                LineageAssetRef(
                    type="TABLE", database="db", schema="s", name="src", asset_id="src1"
                ),
            ],
            fields=[
                ColumnLineageField(
                    name="total",
                    source_fields=[
                        ColumnLineageSourceField(asset_id="src1", field_name="amount"),
                    ],
                ),
            ],
        )
        result: dict[str, Any] = event.to_dict()
        assert len(result["fields"]) == 1
        assert result["fields"][0]["name"] == "total"
        assert result["sources"][0]["asset_id"] == "src1"


class TestBuildLineagePayload(TestCase):
    def _make_table_event(self) -> LineageEvent:
        return LineageEvent(
            destination=LineageAssetRef(
                type="TABLE", database="analytics", schema="pub", name="out"
            ),
            sources=[
                LineageAssetRef(type="TABLE", database="raw", schema="pub", name="orders"),
                LineageAssetRef(type="TABLE", database="raw", schema="pub", name="customers"),
            ],
        )

    def _make_column_event(self) -> LineageEvent:
        return LineageEvent(
            destination=LineageAssetRef(type="TABLE", database="db", schema="s", name="dst"),
            sources=[
                LineageAssetRef(
                    type="TABLE", database="db", schema="s", name="src", asset_id="src1"
                ),
            ],
            fields=[
                ColumnLineageField(
                    name="total",
                    source_fields=[
                        ColumnLineageSourceField(asset_id="src1", field_name="amount"),
                    ],
                ),
            ],
        )

    def test_payload_structure_table_lineage(self):
        payload = build_lineage_payload(
            resource_uuid="res-123",
            resource_type="snowflake",
            events=[self._make_table_event()],
        )
        assert payload["event_type"] == "LINEAGE"
        assert payload["resource"] == {
            "uuid": "res-123",
            "resource_type": "snowflake",
        }
        assert len(payload["events"]) == 1
        assert payload["events"][0]["destination"]["name"] == "out"
        assert len(payload["events"][0]["sources"]) == 2

    def test_auto_detects_column_lineage(self):
        payload = build_lineage_payload(
            resource_uuid="res-123",
            resource_type="snowflake",
            events=[self._make_column_event()],
        )
        assert payload["event_type"] == "COLUMN_LINEAGE"
        assert "fields" in payload["events"][0]

    def test_auto_detects_column_lineage_with_mixed_events(self):
        payload = build_lineage_payload(
            resource_uuid="res-123",
            resource_type="snowflake",
            events=[self._make_table_event(), self._make_column_event()],
        )
        assert payload["event_type"] == "COLUMN_LINEAGE"

    def test_explicit_event_type_enum_overrides_auto_detection(self):
        payload = build_lineage_payload(
            resource_uuid="res-123",
            resource_type="snowflake",
            events=[self._make_column_event()],
            event_type=LineageEventType.LINEAGE,
        )
        assert payload["event_type"] == "LINEAGE"

    def test_explicit_event_type_string_overrides_auto_detection(self):
        payload = build_lineage_payload(
            resource_uuid="res-123",
            resource_type="snowflake",
            events=[self._make_column_event()],
            event_type="LINEAGE",
        )
        assert payload["event_type"] == "LINEAGE"

    def test_lineage_event_type_enum_values(self):
        assert LineageEventType.LINEAGE.value == "LINEAGE"
        assert LineageEventType.COLUMN_LINEAGE.value == "COLUMN_LINEAGE"

    def test_multiple_events(self):
        events = [self._make_table_event() for _ in range(3)]
        payload = build_lineage_payload(
            resource_uuid="res-456",
            resource_type="bigquery",
            events=events,
        )
        assert len(payload["events"]) == 3



#!/usr/bin/env python3
"""
Send lineage events (table-level and column-level) to Monte Carlo using pycarlo.

Prerequisites:
    1. Install pycarlo locally:  pip install -e /path/to/python-sdk
    2. Create an integration key with Ingestion scope:
         montecarlo integrations create-key --scope Ingestion --description "lineage test"
       Or via the GraphQL API / UI.

Usage (table-level lineage):
    python sample_ingest_lineage.py \
        --key-id <INGESTION_KEY_ID> \
        --key-token <INGESTION_KEY_SECRET> \
        --resource-uuid <WAREHOUSE_UUID> \
        --resource-type snowflake \
        --asset-prefix lineage_run_01

Usage (column-level lineage):
    python sample_ingest_lineage.py \
        --key-id <INGESTION_KEY_ID> \
        --key-token <INGESTION_KEY_SECRET> \
        --resource-uuid <WAREHOUSE_UUID> \
        --resource-type snowflake \
        --asset-prefix lineage_run_01 \
        --column-lineage
"""

from __future__ import annotations

import argparse
import json

from pycarlo.core import Client, Session
from pycarlo.features.ingestion import IngestionService
from pycarlo.features.ingestion.models import (
    ColumnLineageField,
    ColumnLineageSourceField,
    LineageAssetRef,
    LineageEvent,
)

DEFAULT_ENDPOINT = "https://integrations.getmontecarlo.com"
DEFAULT_ASSET_PREFIX = "run_01"
MANUAL_INGESTION_DATABASE = "manual_ingestion"
MANUAL_INGESTION_SCHEMA = "test"
SEARCH_LIMIT = 20

GET_TABLE_LINEAGE_QUERY_V2 = """
    query getTableLineageV2(
      $mcons: [String]!
      $direction: LineageGraphTraversalDirection!
      $filters: [LineageFilter]
    ) {
      getTableLineageV2(
        mcons: $mcons
        direction: $direction
        filters: $filters
      ) {
        nodes {
          displayName
          mcon
          objectType
          elementId
        }
        edges {
          source
          destination
          sourceId
          destinationId
          hidden
        }
      }
    }
"""
SEARCH_QUERY = """
    query search($query: String!, $limit: Int!) {
        search(
            query: $query
            limit: $limit
            offset: 0
            fullResults: true
            operator: "AND"
        ) {
            totalHits
            results {
                mcon
                displayName
                objectType
            }
        }
    }
"""


def asset_name(prefix: str, suffix: str) -> str:
    return f"{prefix}_{suffix}"


def build_table_lineage_events(prefix: str) -> list[LineageEvent]:
    """Create sample table lineage aligned with the metadata sample assets."""
    destination_table: str = asset_name(prefix, "c")
    source_a: str = asset_name(prefix, "a")
    source_b: str = asset_name(prefix, "b")
    downstream_view: str = asset_name(prefix, "view_c")
    return [
        LineageEvent(
            destination=LineageAssetRef(
                type="TABLE",
                name=destination_table,
                database=MANUAL_INGESTION_DATABASE,
                schema=MANUAL_INGESTION_SCHEMA,
            ),
            sources=[
                LineageAssetRef(
                    type="TABLE",
                    name=source_a,
                    database=MANUAL_INGESTION_DATABASE,
                    schema=MANUAL_INGESTION_SCHEMA,
                ),
                LineageAssetRef(
                    type="TABLE",
                    name=source_b,
                    database=MANUAL_INGESTION_DATABASE,
                    schema=MANUAL_INGESTION_SCHEMA,
                ),
            ],
        ),
        LineageEvent(
            destination=LineageAssetRef(
                type="VIEW",
                name=downstream_view,
                database=MANUAL_INGESTION_DATABASE,
                schema=MANUAL_INGESTION_SCHEMA,
            ),
            sources=[
                LineageAssetRef(
                    type="TABLE",
                    name=destination_table,
                    database=MANUAL_INGESTION_DATABASE,
                    schema=MANUAL_INGESTION_SCHEMA,
                ),
            ],
        ),
    ]


def build_column_lineage_events(prefix: str) -> list[LineageEvent]:
    """Create sample column lineage aligned with the metadata sample assets."""
    destination_table: str = asset_name(prefix, "c")
    source_a: str = asset_name(prefix, "a")
    source_b: str = asset_name(prefix, "b")
    source_a_asset_id = f"{prefix}_src_a"
    source_b_asset_id = f"{prefix}_src_b"
    destination_asset_id = f"{prefix}_dest_c"

    return [
        LineageEvent(
            destination=LineageAssetRef(
                type="TABLE",
                name=destination_table,
                database=MANUAL_INGESTION_DATABASE,
                schema=MANUAL_INGESTION_SCHEMA,
                asset_id=destination_asset_id,
            ),
            sources=[
                LineageAssetRef(
                    type="TABLE",
                    name=source_a,
                    database=MANUAL_INGESTION_DATABASE,
                    schema=MANUAL_INGESTION_SCHEMA,
                    asset_id=source_a_asset_id,
                ),
                LineageAssetRef(
                    type="TABLE",
                    name=source_b,
                    database=MANUAL_INGESTION_DATABASE,
                    schema=MANUAL_INGESTION_SCHEMA,
                    asset_id=source_b_asset_id,
                ),
            ],
            fields=[
                ColumnLineageField(
                    name="event_id",
                    source_fields=[
                        ColumnLineageSourceField(
                            asset_id=source_a_asset_id,
                            field_name="id",
                        ),
                    ],
                ),
                ColumnLineageField(
                    name="event_type",
                    source_fields=[
                        ColumnLineageSourceField(
                            asset_id=source_b_asset_id,
                            field_name="status",
                        ),
                    ],
                ),
            ],
        ),
    ]


def main() -> None:
    parser = argparse.ArgumentParser(
        description="Send sample lineage to Monte Carlo via pycarlo IngestionService",
    )
    parser.add_argument("--key-id", required=True, help="Ingestion key ID (x-mcd-id)")
    parser.add_argument("--key-token", required=True, help="Ingestion key token (x-mcd-token)")
    parser.add_argument(
        "--endpoint",
        default=DEFAULT_ENDPOINT,
        help=f"Integration Gateway endpoint (default: {DEFAULT_ENDPOINT})",
    )
    parser.add_argument("--resource-uuid", default="your-warehouse-uuid", help="Resource UUID")
    parser.add_argument(
        "--resource-type",
        default="snowflake",
        help="Resource type (e.g. snowflake, bigquery)",
    )
    parser.add_argument(
        "--column-lineage",
        action="store_true",
        help="Send column-level lineage instead of table-level",
    )
    parser.add_argument(
        "--payload-file",
        help="Path to a JSON file with a custom payload (uses send_lineage_raw)",
    )
    parser.add_argument(
        "--asset-prefix",
        default=DEFAULT_ASSET_PREFIX,
        help=(
            "Prefix for the sample assets so metadata and lineage can target a fresh "
            f"shared set (default: {DEFAULT_ASSET_PREFIX})"
        ),
    )
    args = parser.parse_args()

    client = Client(
        session=Session(
            mcd_id=args.key_id,
            mcd_token=args.key_token,
            endpoint=args.endpoint,
            scope="Ingestion",
        )
    )
    service = IngestionService(mc_client=client)

    if args.payload_file:
        with open(args.payload_file, encoding="utf-8") as file_handle:
            payload = json.load(file_handle)
        print(f"Sending raw lineage payload from {args.payload_file} ...")
        result = service.send_lineage_raw(payload=payload)
    else:
        if args.column_lineage:
            events = build_column_lineage_events(args.asset_prefix)
            label = "column-level"
        else:
            events = build_table_lineage_events(args.asset_prefix)
            label = "table-level"

        print(
            "Sample assets: "
            f"{asset_name(args.asset_prefix, 'a')}, "
            f"{asset_name(args.asset_prefix, 'b')}, "
            f"{asset_name(args.asset_prefix, 'c')}, "
            f"{asset_name(args.asset_prefix, 'view_c')}"
        )
        print(
            f"Sending {len(events)} {label} lineage event(s) to resource {args.resource_uuid} ..."
        )
        result = service.send_lineage(
            resource_uuid=args.resource_uuid,
            resource_type=args.resource_type,
            events=events,
        )

    invocation_id = service.extract_invocation_id(result)
    print(f"Response: {json.dumps(result, indent=2) if result else '(empty)'}")
    if invocation_id:
        print(f"Invocation ID: {invocation_id}")
    print("Done.")


if __name__ == "__main__":
    main()
